@@ -130,123 +130,16 @@ hczw <- function(threshold,n,beta){
 # rho - common correlation coefficient.
 # k0 - search range starts from the k0th smallest p-value.
 # k1 - search range ends at the k1th smallest p-value.
-# NODES - the numerical integretion domain (under the standard normal pdf).
+# NODES - the numerical integration domain (under the standard normal pdf).
 # s - the phi-divergence test parameter.
 # t - numerical truncation parameter.
 # onesided - TRUE if the input p-values are one-sided.
-pphi.rho <- function(threshold, n, rho, k0, k1, NODES=seq(-4,4,length.out=40),
-                     s=2, t=30, onesided=T){
-  t = min(t,k1-k0+1)
-  u = sapply(1:k1,function(x)phi.f.inv.i(threshold, x, n, s))
-  um = u[length(u)]
-  #u = phiInverse(threshold, s, k1=beta*n, n, t=beta*n)$bound
-  if(rho!=0){
-    SUM = sum(dnorm(NODES))
-    result = 0
-    if(onesided==T){
-      for(z in NODES){
-        d = pnorm((qnorm(1-u)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-        dm = pnorm((qnorm(1-um)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-        result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-      }
-    }else{
-      for(z in NODES){
-        d = pnorm((qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-          pnorm((-qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho))
-        dm = pnorm((qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-          pnorm((-qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho))
-        result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-      }
-    }
-    return(result)
-  }else{
-    return(UnifCross_v1(u,um,t=t,n=n,k0=k0,k1=k1))
-  }
-}
-#Crossing probablity of uniform order statistics
-UnifCross_v1 <- function(u, um, t, n, k0, k1){
-  #um = boundary_end
-  #u = boundary
-  m = floor(k1)
-  #pp = exp(lfactorial(n) - lfactorial(n-(0:(t-1))))*pbeta(um,M+1,n-m+1,lower.tail=F)
-  pp = rep(NA, t)
-  pp[1] = pbeta(um,m,n-m+1,lower.tail=F)
-  pp[2:t] = exp(lfactorial(n) - lfactorial(n-(k0:(k0+t-2))))*pbeta(um,m-(k0:(k0+t-2)),n-m+1,lower.tail=F)
-  a = rep(1,t)
-  a[2] = -dpois(k0,u[k0])*exp(u[k0])
-  if(t>1){
-    for (i in 2:(t-1)){
-      d = dpois(c(1:(i-1), i+k0-1),u[i+k0-1])*exp(u[i+k0-1])
-
-      a[i+1]=-a[i:1]%*%d
-    }
-    #p = 1-pp[1:t]%*%a[1:t]
-    #if(p>=pbeta(u[2], 2, n-1)-u[1]*n*(1-pbeta(u[2],1,n-1))&&p<=1){
-    #return(pp[1:(t-k0+1)]%*%a[1:(t-k0+1)])
-    return(drop(pp[1:t]%*%a[1:t]))
-    #}else{
-    #warning("q is too small. The algorithm fails due to loss of significant digits", call. = FALSE)
-    #return(0)
-    #}
-  }else{
-    p = pbeta(u[k0], k0, n-k0+1)
-    return(1-drop(p))
-  }
-}
-
-qphi.rho_v1 <- function (p, n, rho,NODES, k0, k1,s,t=28,onesided=T) {
-  lower = -1
-  upper = 10
-  maxnumIter = 500
-  numIter = 1
-  ddn = dnorm(NODES)
-  p_cal_lower = pphi.rho_v1(lower, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-  p_cal_upper = pphi.rho_v1(upper, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-  while ((p_cal_lower - p) * (p_cal_upper - p) > 0 && numIter <
-         maxnumIter) {
-    if (p_cal_lower > p) {
-      lower = lower * 1.1
-      p_cal_lower = pphi.rho_v1(lower, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-    }
-    if (p_cal_upper < p) {
-      upper = upper * 1.1
-      p_cal_upper = pphi.rho_v1(upper, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-    }
-    numIter = numIter + 1
-  }
-  q = mean(c(lower, upper))
-  p_cal = pphi.rho_v1(q, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-  error = (p_cal - p)/p
-  numIter = 1
-  while (abs(error) > 1e-05 && numIter < maxnumIter) {
-    if (error > 0) {
-      upper = q
-      q = mean(c(lower, upper))
-      p_cal = pphi.rho_v1(q, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-      error = (p_cal - p)/p
-    }
-    else {
-      lower = q
-      q = mean(c(lower, upper))
-      p_cal = pphi.rho_v1(q, n, rho,NODES, k0, k1,s,t,onesided, normdn=ddn)
-      error = (p_cal - p)/p
-    }
-    numIter = numIter + 1
-  }
-  if (numIter < maxnumIter) {
-    return(q)
-  }
-  else {
-    return("Bisection fails, choose another lower or upper bound to try again")
-  }
-}
-
-pphi.rho_v1 <- function(threshold, n, rho, NODES, k0, k1, s=2, t=30, onesided=T,normdn=NULL){
+# normdn - dnorm(NODES)
+pphi.rho <- function(threshold, n, rho, k0, k1, s=2, t=30, onesided=F, NODES=seq(-8,8,length.out=50),normdn=NULL){
   nNODES = length(NODES)
   t = min(t,k1-k0+1)
   u = sapply(1:k1,function(x)phi.f.inv.i(threshold, x, n, s))
   um = u[length(u)]
-  #u = phiInverse(threshold, s, k1=beta*n, n, t=beta*n)$bound
   if(is.null(normdn)){
     if(rho!=0){
       SUM = sum(dnorm(NODES))
@@ -256,80 +149,158 @@ pphi.rho_v1 <- function(threshold, n, rho, NODES, k0, k1, s=2, t=30, onesided=T,
         qum = qnorm(1-um)
         sqrho = sqrt(rho)
         sq_rho = sqrt(1-rho)
-        D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F))
+        if(length(qu)==1){
+          D = t(as.matrix(sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F))))
+        }else{
+          D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F))
+        }
+        
         DM = sapply(1:nNODES, function(x)pnorm((qum-sqrho*NODES[x])/sq_rho,lower.tail = F))
-        result = sum(dnorm(NODES)*sapply(1:nNODES, function(x)UnifCross_v1(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
+        result = sum(dnorm(NODES)*sapply(1:nNODES, function(x)UnifCross(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
 
-        # for(z in NODES){
-        #   d = pnorm((qnorm(1-u)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-        #   dm = pnorm((qnorm(1-um)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-        #   result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-        # }
       }else{
         qu = qnorm(1-u/2)
         qum = qnorm(1-um/2)
         sqrho = sqrt(rho)
         sq_rho = sqrt(1-rho)
-        D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F) +
-                     pnorm((-qu-sqrho*NODES[x])/sq_rho))
+        if(length(qu)==1){
+          D = t(as.matrix(sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F) +
+                                   pnorm((-qu-sqrho*NODES[x])/sq_rho))))
+        }else{
+          D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F) +
+                       pnorm((-qu-sqrho*NODES[x])/sq_rho))
+        }
+        
         DM = sapply(1:nNODES, function(x)pnorm((qum-sqrho*NODES[x])/sq_rho,lower.tail = F) +
                       pnorm((-qum-sqrho*NODES[x])/sq_rho))
-        result = sum(dnorm(NODES)*sapply(1:nNODES, function(x)UnifCross_v1(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
-
-        # for(z in NODES){
-        #   d = pnorm((qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-        #     pnorm((-qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho))
-        #   dm = pnorm((qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-        #     pnorm((-qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho))
-        #   result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-        # }
+        
+        result = sum(dnorm(NODES)*sapply(1:nNODES, function(x)UnifCross(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
+        
       }
       return(result)
     }else{
-      return(UnifCross_v1(u,um,t=t,n=n,k0=k0,k1=k1))
+      return(UnifCross(u,um,t=t,n=n,k0=k0,k1=k1))
     }
   }else{
+    # edit new version
     SUM = sum(normdn)
+    sqrho = sqrt(rho)
+    sq_rho = sqrt(1-rho)
+    sqrhoNODES = sqrho*NODES
     result = 0
     if(onesided==T){
       qu = qnorm(1-u)
       qum = qnorm(1-um)
-      sqrho = sqrt(rho)
-      sq_rho = sqrt(1-rho)
-      D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F))
-      DM = sapply(1:nNODES, function(x)pnorm((qum-sqrho*NODES[x])/sq_rho,lower.tail = F))
-      result = sum(normdn*sapply(1:nNODES, function(x)UnifCross_v1(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
-
-      # for(i in 1:nNODES){
-      #   z = NODES[i]
-      #   d = pnorm((qnorm(1-u)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-      #   dm = pnorm((qnorm(1-um)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-      #   result = result + normdn[i]*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-      # }
+      if(length(qu)==1){
+        D = t(as.matrix(sapply(1:nNODES, function(x)pnorm((qu-sqrhoNODES[x])/sq_rho,lower.tail = F))))
+      }else{
+        D = sapply(1:nNODES, function(x)pnorm((qu-sqrhoNODES[x])/sq_rho,lower.tail = F))
+      }
+      
+      DM = sapply(1:nNODES, function(x)pnorm((qum-sqrhoNODES[x])/sq_rho,lower.tail = F))
+      result = sum(normdn*sapply(1:nNODES, function(x)UnifCross(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
     }else{
       qu = qnorm(1-u/2)
       qum = qnorm(1-um/2)
-      sqrho = sqrt(rho)
-      sq_rho = sqrt(1-rho)
-      D = sapply(1:nNODES, function(x)pnorm((qu-sqrho*NODES[x])/sq_rho,lower.tail = F) +
-                   pnorm((-qu-sqrho*NODES[x])/sq_rho))
-      DM = sapply(1:nNODES, function(x)pnorm((qum-sqrho*NODES[x])/sq_rho,lower.tail = F) +
-                    pnorm((-qum-sqrho*NODES[x])/sq_rho))
-      result = sum(normdn*sapply(1:nNODES, function(x)UnifCross_v1(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
-
-      # for(i in 1:nNODES){
-      #   z = NODES[i]
-      #   d = pnorm((qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-      #     pnorm((-qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho))
-      #   dm = pnorm((qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-      #     pnorm((-qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho))
-      #   result = result + normdn[i]*UnifCross_v1(d,dm,t=t,n=n,k0=k0,k1=k1)/SUM
-      # }
+      if(length(qu)==1){
+        D = t(as.matrix(sapply(1:nNODES, function(x)pnorm((qu-sqrhoNODES[x])/sq_rho,lower.tail = F) +
+                                 pnorm((-qu-sqrhoNODES[x])/sq_rho))))
+      }else{
+        D = sapply(1:nNODES, function(x)pnorm((qu-sqrhoNODES[x])/sq_rho,lower.tail = F) +
+                     pnorm((-qu-sqrhoNODES[x])/sq_rho))
+      }
+      DM = sapply(1:nNODES, function(x)pnorm((qum-sqrhoNODES[x])/sq_rho,lower.tail = F) +
+                    pnorm((-qum-sqrhoNODES[x])/sq_rho))
+      result = sum(normdn*sapply(1:nNODES, function(x)UnifCross(D[,x],DM[x],t=t,n=n,k0=k0,k1=k1)))/SUM
     }
     return(result)
   }
 }
 
+#Crossing probablity of uniform order statistics
+UnifCross <- function(u, um, t, n, k0, k1){
+  #um = boundary_end
+  #u = boundary
+  m = floor(k1)
+  #pp = exp(lfactorial(n) - lfactorial(n-(0:(t-1))))*pbeta(um,M+1,n-m+1,lower.tail=F)
+  pp = rep(NA, t)
+  pp[1] = pbeta(um,m,n-m+1,lower.tail=F)
+  pp[2:t] = exp(lfactorial(n) - lfactorial(n-(k0:(k0+t-2))))*pbeta(um,m-(k0:(k0+t-2)),n-m+1,lower.tail=F)
+  a = rep(1,t)
+  a[2] = -dpois(k0,u[k0])*exp(u[k0])
+  if(t>2){
+    for (i in 2:(t-1)){
+      d = dpois(c(1:(i-1), i+k0-1),u[i+k0-1])*exp(u[i+k0-1])
+      # a[i+1]=-a[i:1]%*%d
+      # edit new version
+      a[i+1]=-sum(a[i:1]*d)
+    }
+    #p = 1-pp[1:t]%*%a[1:t]
+    #if(p>=pbeta(u[2], 2, n-1)-u[1]*n*(1-pbeta(u[2],1,n-1))&&p<=1){
+    #return(pp[1:(t-k0+1)]%*%a[1:(t-k0+1)])
+    #return(drop(pp[1:t]%*%a[1:t]))
+    # edit new version
+    finalp = sum(pp*a)
+    #}else{
+    #warning("q is too small. The algorithm fails due to loss of significant digits", call. = FALSE)
+    #return(0)
+    #}
+  }else if(t==2){
+    finalp = sum(pp*a)
+  }else{
+    p = pbeta(u[k0], k0, n-k0+1)
+    finalp = 1-drop(p)
+  }
+  if(finalp>1|finalp<0){
+    p = pbeta(u[k0], k0, n-k0+1)
+    finalp = 1-drop(p)
+  }
+  return(finalp)
+}
+
+qphi.rho <- function (p, n, rho,NODES, k0, k1,s,t=28,onesided=F, err_thr=1e-4) {
+  lower = -1
+  upper = 10
+  maxnumIter = 100
+  numIter = 1
+  ddn = dnorm(NODES)
+  p_cal_lower = pphi.rho(lower, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+  p_cal_upper = pphi.rho(upper, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+  while ((p_cal_lower - p) * (p_cal_upper - p) > 0 && numIter <
+         maxnumIter) {
+    if (p_cal_lower > p) {
+      lower = lower * 1.1
+      p_cal_lower = pphi.rho(lower, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+    }
+    if (p_cal_upper < p) {
+      upper = upper * 1.1
+      p_cal_upper = pphi.rho(upper, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+    }
+    numIter = numIter + 1
+  }
+  q = mean(c(lower, upper))
+  p_cal = pphi.rho(q, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+  error = (p_cal - p)/(1-p)
+  numIter = 1
+  while (abs(error) > err_thr && upper-lower>1e-4 && numIter < maxnumIter) {
+    if (error > 0) {
+      upper = q
+      q = mean(c(lower, upper))
+      p_cal = pphi.rho(q, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+    }
+    else {
+      lower = q
+      q = mean(c(lower, upper))
+      p_cal = pphi.rho(q, n=n, rho=rho,NODES=NODES, k0=k0, k1=k1,s=s,t=t,onesided=onesided, normdn=ddn)
+    }
+    error = (p_cal - p)/(1-p)
+    numIter = numIter + 1
+  }
+  return(q)
+}
+
+
+
 ########################################################################
 # calculate the right-tail probability of omnibus phi-divergence statistics under equal correlation matrix.
 # threshold - threshold.
@@ -338,38 +309,53 @@ pphi.rho_v1 <- function(threshold, n, rho, NODES, k0, k1, s=2, t=30, onesided=T,
 # K0 - vector of search range starts (from the k0th smallest p-value).
 # K1 - vector of search range ends (at the k1th smallest p-value).
 # S - vector of the phi-divergence test parameters.
-# NODES - the numerical integretion domain (under the standard normal pdf).
+# NODES - the numerical integration domain (under the standard normal pdf).
 # t - numerical truncation parameter.
 # onesided - TRUE if the input p-values are one-sided.
-# pphi.rho.omni(0.05, n=10, rho=0.3, K0=rep(1,4), K1=rep(5,4), S=c(-1,0,1,2))
+# pphi.rho.omni(0.05, n=10, rho=0.3, K0=rep(1,2), K1=rep(5,2), S=c(1,2))
 
-pphi.rho.omni <- function(threshold, n, rho, K0, K1, S, NODES=seq(-4,4,length.out=40), t=30, onesided=T){
+pphi.rho.omni <- function(threshold, n, rho, K0, K1, S, NODES=seq(-8,8,length.out=50), t=30, onesided=F){
   t = min(t,max(K1)-min(K0)+1)
   M = floor(K1)
-  Q = mapply(function(x,y,z)qphi.rho_v1(1-threshold, n, rho, NODES, x, y, z, t, onesided),K0,K1,S)
-
+  Q = mapply(function(x,y,z)qphi.rho(p=1-threshold, n=n, rho=rho, NODES=NODES, k0=x, k1=y, s=z, t=t, onesided=onesided),
+             K0,K1,S)
+  
+  id = which(Q=="Bisection fails, choose another lower or upper bound to try again")
+  if(length(id)>0&length(id)<length(S)){
+    Q = as.numeric(Q[-id])
+    K0 = K0[-id]
+    K1 = K1[-id]
+    S = S[-id]
+  }
   boundary =matrix(0, max(K1)-min(K0)+1, length(S))
   for (i in 1:length(S)) {
     boundary[K0[i]:K1[i],i] = mapply(function(x)phi.f.inv.i(Q[i],x,n,S[i]), K0[i]:K1[i])
   }
   u = apply(boundary,1,max)
   um = max(boundary[max(K1),])
-  #u = phiInverse(threshold, s, k1=beta*n, n, t=beta*n)$bound
-  SUM = sum(dnorm(NODES))
+  dnormNODES = dnorm(NODES)
+  dnormz = dnormNODES/sum(dnormNODES)
+  sqrhoz = sqrt(rho)*NODES
+  sqrho_1 = sqrt(1-rho)
   result = 0
   if(onesided==T){
-    for(z in NODES){
-      d = pnorm((qnorm(1-u)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-      dm = pnorm((qnorm(1-um)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F)
-      result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=min(K0),k1=max(K1))/SUM
+    qnormu = qnorm(1-u)
+    qnormum = qnorm(1-um)
+    
+    for(i in 1:length(NODES)){
+      d = pnorm((qnormu-sqrhoz[i])/sqrho_1,lower.tail = F)
+      dm = pnorm((qnormum-sqrhoz[i])/sqrho_1,lower.tail = F)
+      result = result + dnormz[i]*UnifCross(d,dm,t=t,n=n,k0=min(K0),k1=max(K1))
     }
   }else{
-    for(z in NODES){
-      d = pnorm((qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-        pnorm((-qnorm(1-u/2)-sqrt(rho)*z)/sqrt(1-rho))
-      dm = pnorm((qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho),lower.tail = F) +
-        pnorm((-qnorm(1-um/2)-sqrt(rho)*z)/sqrt(1-rho))
-      result = result + dnorm(z)*UnifCross_v1(d,dm,t=t,n=n,k0=min(K0),k1=max(K1))/SUM
+    qnormu = qnorm(1-u/2)
+    qnormum = qnorm(1-um/2)
+    for(i in 1:length(NODES)){
+      d = pnorm((qnormu-sqrhoz[i])/sqrho_1,lower.tail = F) +
+        pnorm((-qnormu-sqrhoz[i])/sqrho_1)
+      dm = pnorm((qnormum-sqrhoz[i])/sqrho_1,lower.tail = F) +
+        pnorm((-qnormum-sqrhoz[i])/sqrho_1)
+      result = result + dnormz[i]*UnifCross(d,dm,t=t,n=n,k0=min(K0),k1=max(K1))
     }
   }
   return(1-result)
